"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const borsh = __importStar(require("@coral-xyz/borsh"));
const CLUSTER = process.env.CLUSTER || 'mainnet-beta';
const CLUSTER_URL = process.env.CLUSTER_URL;
const PAYER_KEYPAIR = process.env.PAYER_KEYPAIR;
async function run() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(PAYER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
    });
    //
    // Retrieve accounts
    //
    const discriminatorMemcmp = client.program.account.mangoAccount.coder.accounts.memcmp('mangoAccount', undefined);
    const accountAis = await client.program.provider.connection.getProgramAccounts(client.programId, {
        filters: [
            {
                memcmp: {
                    bytes: discriminatorMemcmp.bytes,
                    offset: discriminatorMemcmp.offset,
                },
            },
        ],
    });
    console.log('accounts', accountAis.length);
    const accountsToMigrate = accountAis.filter((accountAi) => {
        const version = getAccountVersion(client, accountAi.account);
        return version != 'v3';
    });
    for (let i = 0; i < accountsToMigrate.length; i += 8) {
        const batch = accountsToMigrate.slice(i, i + 8);
        const ixs = await Promise.all(batch.map((accountAi) => makeMigationIx(client, accountAi.pubkey, accountAi.account)));
        await client.sendAndConfirmTransaction(ixs);
    }
}
async function makeMigationIx(client, pubkey, ai) {
    const account = await client.getMangoAccountFromAi(pubkey, ai);
    return await client.program.methods
        .accountSizeMigration()
        .accounts({
        group: account.group,
        account: pubkey,
        payer: client.program.provider.wallet.publicKey,
    })
        .instruction();
}
function getAccountVersion(client, ai) {
    const currentSize = ai.data.length;
    // Decode a v1 account
    const decodedMangoAccount = client.program.coder.accounts.decode('mangoAccount', ai.data);
    // v1: basic length before the introduction of tcs
    const mangoAccountBuffer = Buffer.alloc(currentSize);
    const layout = client.program.coder.accounts['accountLayouts'].get('mangoAccount');
    const discriminatorLen = 8;
    const v1DataLen = layout.encode(decodedMangoAccount, mangoAccountBuffer);
    const v1Len = discriminatorLen + v1DataLen;
    if (currentSize == v1Len) {
        return 'v1';
    }
    // v2: addition of the tcs vector
    const tcsAlign = 4;
    const tcsLayout = client.program._coder.types.typeLayouts.get('TokenConditionalSwap');
    const tcsVecLayout = borsh.vec(tcsLayout);
    const tokenConditionalSwaps = tcsVecLayout.decode(ai.data, v1Len + tcsAlign);
    const tcsBytesSize = tcsVecLayout.encode(tokenConditionalSwaps, mangoAccountBuffer, v1Len + tcsAlign);
    const v2DataLen = v1DataLen + tcsAlign + tcsBytesSize;
    const v2Len = discriminatorLen + v2DataLen;
    if (currentSize == v2Len) {
        return 'v2';
    }
    // v3: add 64 reserved bytes after the tcs vec
    const v3Len = v2Len + 64;
    if (currentSize == v3Len) {
        return 'v3';
    }
    throw new Error(`unexpected mango account size ${currentSize}, expected ${v1Len} or ${v2Len} or ${v3Len}`);
}
run();
